#!/usr/bin/perl -w

# install.pl: 
# - must run as root
# - generate global school-specific parameters (params.pm file)
# - generate password file (passwd)
# - install files into appropriate places
#
# Copyright (C) 2000-2003 Filippo Menczer and the University of Iowa
# Copyright (C) 2003-2004 Filippo Menczer and Indiana University 
#
#    This file is part of the Recruit system
#
#    Recruit is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Recruit is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Recruit; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use Sys::Hostname;
use Socket;

# from Randall Schwarz
sub seedchar {  
    ('a'..'z','A'..'Z','0'..'9','.','/')[rand(64)];
}

# MAIN
open(LOG, ">install.log") or die "Cannot write install.log: $!\n";
select LOG; $| = 1; select STDOUT; $| = 1;
open(OUT, ">params.pm") or die "Cannot write params.pm: $!\n";
print OUT << 'HEAD';
#!/usr/bin/perl -w

# params.pm: shared global school-specific parameters
# This file is part of Recruit and is generated automatically by install.pl
# Copyright (C) 2000-2003 Filippo Menczer and the University of Iowa
# Copyright (C) 2003-2004 Filippo Menczer and Indiana University 

package params;
use strict;
use vars qw(%global);
%global = (
HEAD

print << 'BODY';

Welcome to the install script for the Recruit system scripts.
I will ask you a few questions necessary for the installation.
If you don't know the answers, consult your system/network admin.

BODY

foreach my $fh (*STDOUT, *LOG) {
	print $fh "Make sure you are running this script as root!\n\n";
}

unless (eval "require Mail::Mailer") {
	print "WARNING: Mail::Mailer not found; if you want email notification, install it now!\n\n";
} 

print "The name of your department, for example \n'School of Informatics': ";
chomp($_ = <>); 
print OUT "\tdept => '$_',\n";
print "\n";

print "The name of your university, for example \n'Indiana University': ";
chomp($_ = <>); 
print OUT "\tuniversity => '$_',\n";
print "\n";

print "The name of your recruiting committee chair, for example \n'Jane Doe': ";
chomp($_ = <>); 
print OUT "\tchair => '$_',\n";
print "\n";

print "The URL of your department homepage, for example \n'http://www.soic.indiana.edu/': ";
chomp($_ = <>); 
print OUT "\tdeptURL => '$_',\n";
print "\n";

print "The URL of your department recruiting page, which should include details and contact information. It is used as contact link. For example \n'http://www.soic.indiana.edu/about/hiring/': ";
chomp($_ = <>); 
print OUT "\tadURL => '$_',\n";
print "\n";

print "The email of a recruiting committee administrator, to be used as the 'From' field for emails originated by the Recruit system. This is also used in the default body text of some emails originated by the Recruit system (which can be edited at runtime). It should be a working email. For example \n'hiring\@soic.indiana.edu': ";
my $adminmail = <>;
chomp $adminmail;
print OUT "\tadminmail => '$adminmail',\n";
print "\n";

print << "AREAS";
List the research areas that applicants will be allowed 
to select. Remember they can select multiple areas. 
We recommend using only lowercase and no punctuation. 
Enter one area per line, and an empty line when done: 
AREAS
my @areas;
do {
	chomp($_ = <>);
	push(@areas, "\"$_\"") if length;
} while (length); 
print OUT "\tareas => [";
print(OUT join(",",@areas));
print OUT "],\n";

print "The starting ID for applications, for example '100': ";
chomp($_ = <>); 
print OUT "\tstartid => '$_',\n";
print "\n";

print << "PATH";
Finally we need the path to the recruit system on your web server.
This should be the path of your web server's DocumentRoot followed
by a directory name that is hard to guess. For example the document
root might be '/var/www/html' or '/Library/WebServer/Documents' or
'/home/httpd' etc. This information is available from your sysadmin
or your web server's config files, eg Apache's httpd.conf.
The directory for the recruit system files should contain at least 8
digits that are hard to guess to ensure the privacy of your applicant
data. An example might be 'recruit_569874125'. 

PATH

print "The path to the recruit system data, for example\n",
	"'/var/www/html/recruit_45216985': ";
my $path = <>;
chomp $path; 
print OUT "\tpath => '$path',\n";
die "Error parsing path...\n" unless ($path =~ m|^/.*?(/[^/]*)$|);
die "Error parsing path...\n" unless defined $1;
print OUT "\tURLpath => '$1',\n";
print "\n";

my $mailflag;
if (eval "require Mail::Mailer") {
	$mailflag = 1;
} else {
	$mailflag = 0;
}
foreach my $fh (*STDOUT, *LOG) {
	if ($mailflag) {
		print $fh "Email notification is enabled; a confirmation/test message\n";
		print $fh "will be sent to $adminmail.\n";
	} else {
		print $fh "Email notification is disabled because your Perl installation\n";
		print $fh "is missing the required module Mail::Mailer. Please advise the\n";
		print $fh "committee members that their email notification settings will\n";
		print $fh "not be honored. Should you wish to install the Mail::Mailer\n";
		print $fh "module now, you will have to run this install script again.\n";
	}
}
print OUT "\tmailflag => $mailflag,\n";

# PASSWORD stuff: if user enters empty, we disable password authentication!
print "\nThe password for the recruiting system:\n",
	"this will be shared by all users and provide minumal security (see Recruit issues page).\n",
	"If instead you will use a central authentication server or other system/server\n",
	"dependent authentication mechanism, you can enter an empty password (just hit RETURN)\n",
	"but be warned that in this case Recruit will have NO security of its own!\n",
	"Enter the password (it will appear on the screen in the clear): ";
my $pass = <>;
chomp $pass;
srand(time());
my $secret;
if (length $pass) {
	my $salt = seedchar().seedchar();
	$secret = crypt($pass, $salt);
	print OUT "\tpasswdflag => 1\n";
} else {
	$secret = '';
	print OUT "\tpasswdflag => 0\n";
}
open(PASS, ">passwd") or die "Cannot open passwd file: $!\n";
print PASS $secret;
close PASS;
if (length $pass) {
	foreach my $fh (*STDOUT, *LOG) {
		print $fh "\nThe password ($pass) has been saved in the passwd file.\n";
		print $fh "If you forget, Recruit has no way to recover it.\n";
	}
} else {
	foreach my $fh (*STDOUT, *LOG) {
		print $fh "\nAn empty password has been saved in the passwd file.\n";
		print $fh "Recruit has no authentication of its own!!!\n";
		print $fh "Set up an authentication service before deploying Recruit!!!\n";
	}
}

print OUT << 'TAIL';
);

1;
TAIL

close OUT;
foreach my $fh (*STDOUT, *LOG) {
	print $fh "\nThe shared global variables have been saved to params.pm.\n";
}

print "\nThe cgi-bin path for your web server, for example \n'/home/www/cgi-bin': ";
my $cgipath = <>;
chomp $cgipath; 
$cgipath .= '/recruit';
my $cgiadminpath = $cgipath . '/admin';
print "\nThe user name for the web server, for example \n'www': ";
my $user = <>;
chomp $user;
print "\nThe group name for the web server, for example \n'www': ";
my $group = <>;
chomp $group;

print "\nCreating directories...\n";
mkdir($path, 0775);
mkdir($cgipath, 0775);
mkdir($cgiadminpath, 0775);
system("chown $user.$group $path"); 
system("chown $user.$group $cgipath");
system("chown $user.$group $cgiadminpath");
print "Installing files...\n";
my @files = qw(index.cgi submit.cgi params.pm);
my @adminfiles = qw(admin.cgi register.cgi review.cgi passwd params.pm);
foreach my $f (@files) {
	system("cp $f $cgipath");
	system("chown $user.$group $cgipath/$f");
	chmod(0750, "$cgipath/$f");
}
foreach my $f (@adminfiles) {
	system("cp $f $cgiadminpath");
	system("chown $user.$group $cgiadminpath/$f");
	chmod(0750, "$cgiadminpath/$f");
}

my $host =  hostname() or die "cannot get server's hostname: $!";
unless ($host eq 'localhost') {
	my $addr = gethostbyname($host) or die "cannot resolve $host: $!";
	$host = gethostbyaddr($addr, AF_INET) or die "cannot re-resolve $addr: $!";
}

if ($mailflag) { 
	print "Sending confirmation email...\n";
	eval {
		Mail::Mailer->import();
		my $mailer = Mail::Mailer->new;
		$mailer->open({To => $adminmail, Subject => "Recruit confirmation"});
		print $mailer "You have been named the recruit committee administrator.\n\n";
		print $mailer "Recruit system URL:      http://$host/cgi-bin/recruit/index.cgi\n";
		print $mailer "Recruit system password: $pass\n";
		print $mailer "Recruit system CGI dir:  $cgipath\n";
		print $mailer "Recruit system data dir: $path\n";
		print $mailer "Recruit system home:     http://www.informatics.indiana.edu/fil/Recruit/\n";
		print $mailer "\nRemember to back-up the recruit system directories!\n";
		$mailer->close() or die "Cannor close mailer: $!\n";	
	};
	if ($@) {
		foreach my $fh (*STDOUT, *LOG) {
			print $fh "Sending confirmation email failed. You may need to configure\n";
			print $fh "sendmail properly in order for email notification to work.\n";
		}
	} else {
		foreach my $fh (*STDOUT, *LOG) {
			print $fh "Email notification seems to work.\n";
		}
	}
}

print << "DONE";

Done. The Recruit system should now be available at: 
  http://$host/cgi-bin/recruit/index.cgi

The application databases should be stored in the two directories:
  $path
  $cgiadminpath
  
Remember to back up these directories. When your recruiting cycle
is over you can just delete these directories, but it is strongly 
recommended to first archive them for future reference.

If you need to set up the Recruit system again in the future,
you can run the install script again.

For future reference, this output has been saved in install.log.

DONE

print LOG << "DONELOG";

The Recruit system should now be available at: 
  http://$host/cgi-bin/recruit/index.cgi

The application databases will be stored in the two directories:
  $path
  $cgiadminpath
  
Remember to back up these directories. When your recruiting cycle
is over you can just delete these directories, but it is strongly 
recommended to first archive them for future reference.

If you need to set up the Recruit system again in the future,
you can run the install script again.

DONELOG

close LOG;
